export declare const objectValidationRemarkKey: unique symbol;
export declare type ValidationRemarks = string | {
    [objectValidationRemarkKey]?: string;
    [key: string]: ValidationRemarks;
};
export declare const getTransformationSchemaKey: unique symbol;
export declare const optionalSchemaKey: unique symbol;
declare const unionTypeKey: unique symbol;
export declare type UnionType<T extends any[]> = {
    [unionTypeKey]: never;
};
export declare type SupportedValueTypes = boolean | number | string | undefined;
export declare type TransformationSchema<T> = undefined extends Extract<T, undefined> ? [T] extends [SupportedValueTypes] ? OptionalTransformationSchema<Exclude<T, undefined>> | ValueTransformationSchema<T> : OptionalTransformationSchema<Exclude<T, undefined>> : [T] extends [SupportedValueTypes] ? ValueTransformationSchema<T> : T extends (infer U)[] ? ArrayTransformationSchema<U> : T extends UnionType<infer U> ? UnionTypeTransformationSchema<U> : ObjectTransformationSchema<T>;
export declare type ValueTransformationSchema<T extends SupportedValueTypes> = (value: unknown) => [T, string | undefined];
export declare type ArrayTransformationSchema<T> = [TransformationSchema<T>];
declare const wrapperKey: unique symbol;
declare type Wrapper<T> = {
    [wrapperKey]: never;
};
declare type WrapOptional<T> = {
    [K in keyof T]: undefined extends Extract<T[K], undefined> ? Wrapper<T[K]> : T[K];
};
declare type RemoveOptional<T> = {
    [K in keyof T]-?: T[K];
};
declare type RemoveWrap<T> = {
    [K in keyof T]: T[K] extends Wrapper<infer U> ? U : T[K];
};
declare type TransformOptionalToUndefinedUnion<T> = RemoveWrap<RemoveOptional<WrapOptional<T>>>;
declare type ObjectToTransformationSchema<T> = {
    [K in keyof T]: TransformationSchema<T[K]>;
};
export declare type ObjectTransformationSchema<T> = ObjectToTransformationSchema<TransformOptionalToUndefinedUnion<T>>;
export declare type OptionalTransformationSchema<T> = {
    readonly [optionalSchemaKey]: TransformationSchema<Exclude<T, undefined>>;
};
declare type MapTupleToTransformationSchemas<Tuple extends [...any[]]> = {
    [Index in keyof Tuple]: TransformationSchema<Tuple[Index]>;
} & {
    length: Tuple['length'];
};
export declare type TupleToUnionTypeTransformationSchemaFunction<T extends any[]> = (base: any) => TupleToUnion<MapTupleToTransformationSchemas<T>>;
export declare type UnionTypeTransformationSchema<T extends any[]> = {
    [getTransformationSchemaKey]: TupleToUnionTypeTransformationSchemaFunction<T>;
};
export declare type DeepWithoutUnionTypes<T> = T extends SupportedValueTypes ? T : undefined extends Extract<T, undefined> ? DeepWithoutUnionTypes<Exclude<T, undefined>> | undefined : T extends (infer U)[] ? DeepWithoutUnionTypes<U>[] : T extends UnionType<infer U> ? DeepWithoutUnionTypes<TupleToUnion<U>> : {
    [K in keyof T]: DeepWithoutUnionTypes<T[K]>;
};
declare type TupleToUnion<T extends any[]> = T extends [infer T1] ? T1 : T extends [infer T1, infer T2] ? T1 | T2 : T extends [infer T1, infer T2, infer T3] ? T1 | T2 | T3 : T extends [infer T1, infer T2, infer T3, infer T4] ? T1 | T2 | T3 | T4 : T extends [infer T1, infer T2, infer T3, infer T4, infer T5] ? T1 | T2 | T3 | T4 | T5 : T extends [infer T1, infer T2, infer T3, infer T4, infer T5, infer T6] ? T1 | T2 | T3 | T4 | T5 | T6 : T extends [
    infer T1,
    infer T2,
    infer T3,
    infer T4,
    infer T5,
    infer T6,
    infer T7
] ? T1 | T2 | T3 | T4 | T5 | T6 | T7 : T extends [
    infer T1,
    infer T2,
    infer T3,
    infer T4,
    infer T5,
    infer T6,
    infer T7,
    infer T8
] ? T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 : T extends [
    infer T1,
    infer T2,
    infer T3,
    infer T4,
    infer T5,
    infer T6,
    infer T7,
    infer T8,
    infer T9
] ? T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 | T9 : never;
export {};
